# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from rest_framework.viewsets import ModelViewSet

from test_project.one_to_one.models import Restaurant
from url_filter.filtersets import ModelFilterSet
from url_filter.integrations.drf_coreapi import CoreAPIURLFilterBackend


class TestCoreAPIURLFilterBackend(object):
    def test_get_schema_fields(self):
        class FilterSet(ModelFilterSet):
            class Meta(object):
                model = Restaurant
                extra_kwargs = {
                    'place': {
                        'id': {'no_lookup': True},
                    },
                }

        class ViewSet(ModelViewSet):
            model = Restaurant
            queryset = Restaurant.objects
            filter_backend = CoreAPIURLFilterBackend
            filter_class = FilterSet

        view = ViewSet()
        fields = view.filter_backend().get_schema_fields(view)

        assert set(i.name for i in fields) == {
            'place__address',
            'place__address__contains',
            'place__address__day',
            'place__address__endswith',
            'place__address__exact',
            'place__address__gt',
            'place__address__gte',
            'place__address__hour',
            'place__address__icontains',
            'place__address__iendswith',
            'place__address__iexact',
            'place__address__in',
            'place__address__iregex',
            'place__address__isnull',
            'place__address__istartswith',
            'place__address__lt',
            'place__address__lte',
            'place__address__minute',
            'place__address__month',
            'place__address__range',
            'place__address__regex',
            'place__address__second',
            'place__address__startswith',
            'place__address__week_day',
            'place__address__year',
            'place__id',
            'place__name',
            'place__name__contains',
            'place__name__day',
            'place__name__endswith',
            'place__name__exact',
            'place__name__gt',
            'place__name__gte',
            'place__name__hour',
            'place__name__icontains',
            'place__name__iendswith',
            'place__name__iexact',
            'place__name__in',
            'place__name__iregex',
            'place__name__isnull',
            'place__name__istartswith',
            'place__name__lt',
            'place__name__lte',
            'place__name__minute',
            'place__name__month',
            'place__name__range',
            'place__name__regex',
            'place__name__second',
            'place__name__startswith',
            'place__name__week_day',
            'place__name__year',
            'serves_hot_dogs',
            'serves_hot_dogs__contains',
            'serves_hot_dogs__day',
            'serves_hot_dogs__endswith',
            'serves_hot_dogs__exact',
            'serves_hot_dogs__gt',
            'serves_hot_dogs__gte',
            'serves_hot_dogs__hour',
            'serves_hot_dogs__icontains',
            'serves_hot_dogs__iendswith',
            'serves_hot_dogs__iexact',
            'serves_hot_dogs__in',
            'serves_hot_dogs__iregex',
            'serves_hot_dogs__isnull',
            'serves_hot_dogs__istartswith',
            'serves_hot_dogs__lt',
            'serves_hot_dogs__lte',
            'serves_hot_dogs__minute',
            'serves_hot_dogs__month',
            'serves_hot_dogs__range',
            'serves_hot_dogs__regex',
            'serves_hot_dogs__second',
            'serves_hot_dogs__startswith',
            'serves_hot_dogs__week_day',
            'serves_hot_dogs__year',
            'serves_pizza',
            'serves_pizza__contains',
            'serves_pizza__day',
            'serves_pizza__endswith',
            'serves_pizza__exact',
            'serves_pizza__gt',
            'serves_pizza__gte',
            'serves_pizza__hour',
            'serves_pizza__icontains',
            'serves_pizza__iendswith',
            'serves_pizza__iexact',
            'serves_pizza__in',
            'serves_pizza__iregex',
            'serves_pizza__isnull',
            'serves_pizza__istartswith',
            'serves_pizza__lt',
            'serves_pizza__lte',
            'serves_pizza__minute',
            'serves_pizza__month',
            'serves_pizza__range',
            'serves_pizza__regex',
            'serves_pizza__second',
            'serves_pizza__startswith',
            'serves_pizza__week_day',
            'serves_pizza__year',
            'waiter__id',
            'waiter__id__contains',
            'waiter__id__day',
            'waiter__id__endswith',
            'waiter__id__exact',
            'waiter__id__gt',
            'waiter__id__gte',
            'waiter__id__hour',
            'waiter__id__icontains',
            'waiter__id__iendswith',
            'waiter__id__iexact',
            'waiter__id__in',
            'waiter__id__iregex',
            'waiter__id__isnull',
            'waiter__id__istartswith',
            'waiter__id__lt',
            'waiter__id__lte',
            'waiter__id__minute',
            'waiter__id__month',
            'waiter__id__range',
            'waiter__id__regex',
            'waiter__id__second',
            'waiter__id__startswith',
            'waiter__id__week_day',
            'waiter__id__year',
            'waiter__name',
            'waiter__name__contains',
            'waiter__name__day',
            'waiter__name__endswith',
            'waiter__name__exact',
            'waiter__name__gt',
            'waiter__name__gte',
            'waiter__name__hour',
            'waiter__name__icontains',
            'waiter__name__iendswith',
            'waiter__name__iexact',
            'waiter__name__in',
            'waiter__name__iregex',
            'waiter__name__isnull',
            'waiter__name__istartswith',
            'waiter__name__lt',
            'waiter__name__lte',
            'waiter__name__minute',
            'waiter__name__month',
            'waiter__name__range',
            'waiter__name__regex',
            'waiter__name__second',
            'waiter__name__startswith',
            'waiter__name__week_day',
            'waiter__name__year',
        }



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import pytest
from django.conf import settings
from django.core.management import call_command
from sqlalchemy.orm import sessionmaker


@pytest.fixture
def one_to_one(db):
    call_command('loaddata', 'one_to_one.json')


@pytest.fixture
def many_to_one(db):
    call_command('loaddata', 'many_to_one.json')


@pytest.fixture
def many_to_many(db):
    call_command('loaddata', 'many_to_many.json')


@pytest.fixture
def alchemy_db(request):
    session = sessionmaker(bind=settings.SQLALCHEMY_ENGINE)()

    def fin():
        session.close()

    request.addfinalizer(fin)

    return session



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import pytest
from django import forms

from url_filter.fields import MultipleValuesField
from url_filter.validators import MaxLengthValidator, MinLengthValidator


class TestMultipleValuesField(object):
    def test_init(self):
        field = MultipleValuesField(
            child=forms.IntegerField(),
            min_values=2,
            max_values=100,
            delimiter=';'
        )

        assert isinstance(field.child, forms.IntegerField)
        assert field.delimiter == ';'
        assert any((isinstance(i, MinLengthValidator) for i in field.many_validators))
        assert any((isinstance(i, MaxLengthValidator) for i in field.many_validators))

    def test_clean_empty(self):
        assert MultipleValuesField(required=False).clean('') is None

    def test_clean(self):
        field = MultipleValuesField(min_values=2, max_values=3)

        assert field.clean('hello,world') == ['hello', 'world']

        with pytest.raises(forms.ValidationError):
            field.clean('hello')
        with pytest.raises(forms.ValidationError):
            field.clean('hello,world,and,many,happy,rainbows')

    def test_clean_all_valid(self):
        field = MultipleValuesField(forms.IntegerField(), all_valid=False)

        assert field.clean('1,2,3') == [1, 2, 3]
        assert field.clean('a,1,b,2') == [1, 2]

    def test_many_to_python(self):
        field = MultipleValuesField()

        assert field.many_to_python('hello,world') == ['hello', 'world']

    def test_many_validate(self):
        assert MultipleValuesField().many_validate([1, 2]) is None
        with pytest.raises(forms.ValidationError):
            MultipleValuesField().many_validate([])

    def test_many_run_validators(self):
        field = MultipleValuesField(error_messages={'min_length': 'foo'})

        assert field.many_run_validators(None) is None

        with pytest.raises(forms.ValidationError) as e:
            field.many_run_validators(['hello'])
        assert e.value.error_list[0].message == 'foo'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import abc

import six
from cached_property import cached_property


class BaseFilterBackend(six.with_metaclass(abc.ABCMeta, object)):
    """
    Base filter backend from which all other backends must subclass.

    Parameters
    ----------
    queryset
        Iterable which this filter backend will eventually filter.
        The type of the iterable depends on the filter backend.
        For example for :class:`.DjangoFilterBackend`, Django's
        ``QuerySet`` needs to be passed.
    context: dict
        Context dictionary. It could contain any information
        which potentially could be useful to filter given
        queryset. That can include, request, view, view kwargs, etc.
        The idea is similar to DRF serializers. By passing the context,
        it allows custom filters to reference all the information
        they need to be able to effectively filter data.
    """
    name = None
    """
    Name of the filter backend.

    This is used by custom callable filters to define callables
    for each supported backend. More at :class:`.CallableFilter`
    """
    supported_lookups = set()
    """
    Set of supported lookups this filter backend supports.

    This is used by leaf :class:`.Filter` to determine whether
    it should construct :class:`.FilterSpec` for a particular
    key-value pair from querystring since it if constructs
    specification but then filter backend will not be able to
    filter it, things will blow up. By explicitly checking
    if filter backend supports particular lookup it can
    short-circuit the logic and avoid errors down the road.
    This is pretty much the only coupling between filters
    and filter backends.
    """
    enforce_same_models = True
    """
    Whether same models should be enforced when trying to use
    this filter backend.

    More can be found in :meth:`BaseFilterBackend.model`
    """

    def __init__(self, queryset, context=None):
        self.queryset = queryset
        self.context = context or {}
        self.specs = []

    @cached_property
    def model(self):
        """
        Property for getting model on which this filter backend operates.

        This is meant to be used by the integrations directly shipped with
        django-url-filter which need to be able to validate that the filterset
        will be able to filter given queryset. They can do that by comparing
        the model they are trying to filter matches the model the filterbackend
        got. This primarily will have misconfigurations such as using
        SQLAlchemy filterset to filter Django's ``QuerySet``.
        """
        return self.get_model()

    def bind(self, specs):
        """
        Bind the given specs to the filter backend.

        This allows the filter backend to be instantiated first before
        filter specs are constructed and later, specs can be binded
        to the backend.

        Parameters
        ----------
        specs : list
            List of :class:`.FilterSpec` to be binded for the filter
            backend for filtering
        """
        self.specs = specs

    @cached_property
    def regular_specs(self):
        """
        Property for getting standard filter specifications
        which can be used directly by the filter backend
        to filter queryset.

        See Also
        --------
        callable_specs
        """
        return [i for i in self.specs if not i.is_callable]

    @cached_property
    def callable_specs(self):
        """
        Property for getting custom filter specifications
        which have a filter callable for filtering querysets.
        These specifications cannot be directly used by filter
        backend and have to be called manually to filter data.

        See Also
        --------
        regular_specs
        """
        return [i for i in self.specs if i.is_callable]

    @abc.abstractmethod
    def get_model(self):
        """
        Get the queryset model.

        .. note:: **MUST** be implemented by subclasses.
            :meth:`.model` property  uses this method to get the model.

        See Also
        --------
        model
        """

    def filter(self):
        """
        Main public method for filtering querysets.
        """
        qs = self.filter_by_specs(self.queryset)
        qs = self.filter_by_callables(qs)
        return qs

    @abc.abstractmethod
    def filter_by_specs(self, queryset):
        """
        Method for filtering queryset by using standard filter specs.

        .. note:: **MUST** be implemented by subclasses
        """

    @abc.abstractmethod
    def empty(self):
        """
        Method for returning empty queryset when any validations failed.
        """

    def filter_by_callables(self, queryset):
        """
        Method for filtering queryset by using custom filter callables
        as given in the :class:`.Filter` definition.

        This is really meant to accommodate filtering with simple
        filter keys having complex filtering logic behind them.
        More about custom callables can be found at :class:`.CallableFilter`
        """
        if not self.callable_specs:
            return queryset

        for spec in self.callable_specs:
            queryset = spec.filter_callable(
                queryset=queryset,
                spec=spec,
            )

        return queryset



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
from functools import partial

import mock
import pytest
from django import forms

from url_filter.backends.django import DjangoFilterBackend
from url_filter.fields import MultipleValuesField
from url_filter.filters import (
    CallableFilter,
    Filter as _Filter,
    form_field_for_filter,
)
from url_filter.utils import FilterSpec, LookupConfig


Filter = partial(_Filter, lookups=DjangoFilterBackend.supported_lookups)


class TestFilter(object):
    def test_init(self):
        f = Filter(
            source='foo',
            form_field=forms.CharField(),
            lookups=['foo', 'bar'],
            default_lookup='foo',
            is_default=True,
        )

        assert f.source == 'foo'
        assert isinstance(f.form_field, forms.CharField)
        assert f.lookups == {'foo', 'bar'}
        assert f.default_lookup == 'foo'
        assert f.is_default is True
        assert f.parent is None
        assert f.name is None

    def test_lookups(self):
        assert Filter(form_field=None, lookups=['foo', 'bar']).lookups == {'foo', 'bar'}
        assert Filter(form_field=None, lookups=None).lookups == set()

        f = Filter(form_field=None, lookups=None)
        f.parent = mock.Mock()
        f.parent.root = f.parent
        f.parent.filter_backend.supported_lookups = DjangoFilterBackend.supported_lookups

        assert f.lookups == DjangoFilterBackend.supported_lookups

    def test_repr(self):
        f = Filter(
            source='foo',
            lookups=None,
            form_field=forms.CharField(),
            default_lookup='foo',
            is_default=True,
        )

        assert repr(f) == (
            'Filter(form_field=CharField, lookups=ALL, '
            'default_lookup="foo", no_lookup=False)'
        )

        f.is_bound = True
        assert repr(f) == (
            'Filter(source="foo", form_field=CharField, lookups=ALL, '
            'default_lookup="foo", is_default=True, no_lookup=False)'
        )

    def test_source(self):
        f = Filter(source=None, form_field=forms.CharField())
        f.name = 'bar'

        assert f.source == 'bar'
        assert Filter(source='foo', form_field=forms.CharField()).source == 'foo'

    def test_components(self):
        p = Filter(source='parent', form_field=forms.CharField())
        f = Filter(source='child', form_field=forms.CharField())

        assert f.components == []
        f.parent = p
        assert f.components == ['child']

    def test_bind(self):
        f = Filter(form_field=forms.CharField())
        f.bind('foo', 'parent')

        assert f.name == 'foo'
        assert f.parent == 'parent'

    def test_root(self):
        p = Filter(source='parent', form_field=forms.CharField())
        f = Filter(source='child', form_field=forms.CharField())
        f.parent = p

        assert f.root is p

    def test_get_form_field(self):
        f = Filter(form_field=forms.CharField())

        assert isinstance(f.get_form_field('exact'), forms.CharField)
        assert isinstance(f.get_form_field('in'), MultipleValuesField)
        assert isinstance(f.get_form_field('isnull'), forms.BooleanField)

    def test_clean_value(self):
        f = Filter(form_field=forms.IntegerField())

        assert f.clean_value('5', 'exact') == 5
        assert f.clean_value('1,2', 'in') == [1, 2]

        with pytest.raises(forms.ValidationError):
            f.clean_value('a', 'exact')

        with pytest.raises(forms.ValidationError):
            f.clean_value('1,a,2,b', 'in')

    def test_get_spec(self):
        p = Filter(source='parent', form_field=forms.CharField())
        f = Filter(source='child', form_field=forms.CharField())
        f.parent = p

        assert f.get_spec(LookupConfig('key', 'value')) == FilterSpec(
            ['child'], 'exact', 'value', False
        )
        assert f.get_spec(LookupConfig('key!', 'value')) == FilterSpec(
            ['child'], 'exact', 'value', True
        )
        assert f.get_spec(LookupConfig('key', {'contains': 'value'})) == FilterSpec(
            ['child'], 'contains', 'value', False
        )

        with pytest.raises(forms.ValidationError):
            assert f.get_spec(LookupConfig('key', {'foo': 'value'}))
        with pytest.raises(forms.ValidationError):
            assert f.get_spec(LookupConfig('key', {
                'foo': 'value', 'happy': 'rainbows',
            }))
        with pytest.raises(forms.ValidationError):
            f.no_lookup = True
            assert f.get_spec(LookupConfig('key', {'exact': 'value'}))


def test_form_field_for_filter():
    field = forms.CharField()

    class Foo(object):
        def foo(self):
            """foo"""
            return 5

        bar = form_field_for_filter(field)(foo)

    f = Foo()
    assert f.bar.__doc__ == f.foo.__doc__
    assert f.bar() == 5
    assert f.bar.form_field is field


class TestCallableFilter(object):
    def test_init(self):
        f = CallableFilter()

        assert f.form_field is None

    def test_lookups(self):
        class Foo(CallableFilter):
            def filter_foo_for_django(self):
                pass

        f = Foo()
        f.filter_backend = mock.Mock(supported_lookups=set())
        f.filter_backend.name = 'django'

        assert f.lookups == {'foo'}

    def test_lookups_not_all_backends(self):
        class Foo(CallableFilter):
            def filter_foo_for_django(self):
                pass

        f = Foo()
        f.filter_backend = mock.Mock(supported_lookups=set())
        f.filter_backend.name = 'sqlalchemy'

        assert f.lookups == set()

    def test_get_form_field(self):
        field = forms.CharField()

        class Foo(CallableFilter):
            @form_field_for_filter(field)
            def filter_foo_for_django(self):
                pass

        f = Foo()
        f.filter_backend = DjangoFilterBackend(queryset=[])

        assert f.get_form_field('foo') is field

    def test_get_form_field_default_form_field(self):
        field = forms.CharField()

        class Foo(CallableFilter):
            def filter_foo_for_django(self):
                pass

        f = Foo(form_field=field, lookups=['exact'])
        f.filter_backend = DjangoFilterBackend(queryset=[])

        assert f.get_form_field('foo') is field

    def test_get_form_field_no_form_field(self):
        class Foo(CallableFilter):
            def filter_foo_for_django(self):
                pass

        f = Foo()
        f.filter_backend = DjangoFilterBackend(queryset=[])

        with pytest.raises(AssertionError):
            f.get_form_field('foo')

    def test_get_spec(self):
        class Foo(CallableFilter):
            @form_field_for_filter(forms.CharField())
            def filter_foo_for_django(self):
                pass

        p = Filter(source='parent', form_field=forms.CharField())
        p.filter_backend = DjangoFilterBackend(queryset=[])
        f = Foo(source='child', default_lookup='foo')
        f.parent = p

        assert f.get_spec(LookupConfig('key', 'value')) == FilterSpec(
            ['child'], 'foo', 'value', False, f.filter_foo_for_django
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from django.core.exceptions import ValidationError as DjangoValidationError
from rest_framework.exceptions import ValidationError
from rest_framework.filters import BaseFilterBackend

from ..filtersets import ModelFilterSet


class DjangoFilterBackend(BaseFilterBackend):
    """
    DRF filter backend which integrates with ``django-url-filter``

    This integration backend can be specified in global DRF settings::

        # settings.py
        REST_FRAMEWORK = {
            'DEFAULT_FILTER_BACKENDS': [
                'url_filter.integrations.drf.DjangoFilterBackend',
            ]
        }

    Alternatively filter backend can be specified per view/viewset bases::

        class MyViewSet(ModelViewSet):
            queryset = MyModel.objects.all()
            filter_backends = [DjangoFilterBackend]
            filter_fields = ['field1', 'field2']

    The following attributes can be specified on the view:

    * ``filter_class`` - explicit filter (:class:`.FilterSet` to be specific) class
      which should be used for filtering. When this attribute is supplied, this
      filterset will be used and all other attributes described below will are ignored.
    * ``filter_fields`` - list of strings which should be names
      of fields which should be included in the generated :class:`.FilterSet`.
      This is equivalent::

          class MyFilterSet(ModelFilterSet):
              class Meta(object):
                  model = MyModel
                  fields = ['fields1', ...]
    * ``filter_class_meta_kwargs`` - additional kwargs which should be passed
      in ``Meta`` for the generated :class:`.FilterSet`.
    * ``filter_class_default`` - base class to use while creating new :class:`.FilterSet`.
      This is primarily useful when using non-Django data-sources.
      By default :attr:`.default_filter_set` is used.

    See Also
    --------
    :py:class:`url_filter.integrations.drf_coreapi.CoreAPIURLFilterBackend`
    """
    default_filter_set = ModelFilterSet
    """
    Default base class which will be used while dynamically creating :class:`.FilterSet`
    """

    def get_filter_class(self, view, queryset=None):
        """
        Get filter class which will be used for filtering.

        Parameters
        ----------
        view : View
            DRF view/viewset where this filter backend is being used.
            Please refer to :class:`.DjangoFilterBackend` documentation
            for list of attributes which can be supplied in view
            to customize how filterset will be determined.
        queryset
            Query object for filtering

        Returns
        -------
        :class:`.FilterSet`
            :class:`.FilterSet` class either directly specified in the view or
            dynamically constructed for the queryset model.
        None
            When appropriate :class:`.FilterSet` cannot be determined
            for filtering
        """
        filter_class_default = getattr(view, 'filter_class_default', self.default_filter_set)
        filter_class = getattr(view, 'filter_class', None)
        filter_class_meta_kwargs = getattr(view, 'filter_class_meta_kwargs', {})
        filter_fields = getattr(view, 'filter_fields', None)

        if filter_class:
            return filter_class

        if filter_fields:
            model = filter_class_default.filter_backend_class(queryset).get_model()

            meta_kwargs = filter_class_meta_kwargs.copy()
            meta_kwargs.update({
                'model': model,
                'fields': filter_fields,
            })
            meta = type(str('Meta'), (object,), meta_kwargs)

            return type(
                str('{}FilterSet'.format(model.__name__)),
                (filter_class_default,),
                {'Meta': meta}
            )

    def get_filter_context(self, request, view):
        """
        Get context to be passed to :class:`.FilterSet` during initialization

        Parameters
        ----------
        request : HttpRequest
            Request object from the view
        view : View
            View where this filter backend is being used

        Returns
        -------
        dict
            Context to be passed to :class:`.FilterSet`
        """
        return {
            'request': request,
            'view': view,
        }

    def filter_queryset(self, request, queryset, view):
        """
        Main method for filtering query object

        Parameters
        ----------
        request : HttpRequest
            Request object from the view
        queryset
            Query object for filtering
        view : View
            View where this filter backend is being used

        Returns
        -------
        object
            Filtered query object if filtering class was determined by
            :meth:`.get_filter_class`. If not given ``queryset`` is returned.
        """
        filter_class = self.get_filter_class(view, queryset)

        if filter_class:
            _filter = filter_class(
                data=request.query_params,
                queryset=queryset,
                context=self.get_filter_context(request, view),
            )

            filter_model = getattr(_filter.Meta, 'model', None)
            if filter_model and _filter.filter_backend.enforce_same_models:
                model = _filter.filter_backend.model
                assert issubclass(model, filter_model), (
                    'FilterSet model {} does not match queryset model {}'
                    ''.format(filter_model, model)
                )

            try:
                return _filter.filter()
            except DjangoValidationError as e:
                raise ValidationError(e.message_dict)

        return queryset


URLFilterBackend = DjangoFilterBackend



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import pytest
from django import forms
from django.http import QueryDict

from test_project.one_to_one.models import Restaurant, Waiter
from url_filter.backends.django import DjangoFilterBackend
from url_filter.constants import StrictMode
from url_filter.exceptions import Empty
from url_filter.filters import Filter
from url_filter.filtersets.base import FilterSet
from url_filter.utils import FilterSpec


class TestFilterSet(object):
    def test_init(self):
        fs = FilterSet(
            data='some data',
            queryset='queryset',
            context={'context': 'here'},
            strict_mode=StrictMode.fail,
        )

        assert fs.data == 'some data'
        assert fs.queryset == 'queryset'
        assert fs.context == {'context': 'here'}
        assert fs.strict_mode == StrictMode.fail

    def test_repr(self):
        class FooFilterSet(FilterSet):
            foo = Filter(form_field=forms.CharField())

        class BarFilterSet(FilterSet):
            bar = Filter(form_field=forms.IntegerField())
            foo = FooFilterSet()

        assert repr(BarFilterSet()) == (
            'BarFilterSet()\n'
            '  bar = Filter(source="bar", form_field=IntegerField, lookups=ALL, default_lookup="exact", is_default=False, no_lookup=False)\n'
            '  foo = FooFilterSet(source="foo")\n'
            '    foo = Filter(source="foo", form_field=CharField, lookups=ALL, default_lookup="exact", is_default=False, no_lookup=False)'
        )

    def test_get_filters(self):
        class TestFilterSet(FilterSet):
            foo = Filter(form_field=forms.CharField())

        filters = TestFilterSet().get_filters()

        assert isinstance(filters, dict)
        assert list(filters.keys()) == ['foo']
        assert isinstance(filters['foo'], Filter)
        assert filters['foo'].parent is None

    def test_filters(self):
        class TestFilterSet(FilterSet):
            foo = Filter(form_field=forms.CharField())

        fs = TestFilterSet()
        filters = fs.filters

        assert isinstance(filters, dict)
        assert list(filters.keys()) == ['foo']
        assert isinstance(filters['foo'], Filter)
        assert filters['foo'].parent is fs
        assert filters['foo'].name == 'foo'

    def test_default_filter_no_default(self):
        class TestFilterSet(FilterSet):
            foo = Filter(form_field=forms.CharField())

        assert TestFilterSet().default_filter is None

    def test_default_filter(self):
        class TestFilterSet(FilterSet):
            foo = Filter(form_field=forms.CharField(), is_default=True)
            bar = Filter(form_field=forms.CharField())

        default = TestFilterSet().default_filter

        assert isinstance(default, Filter)
        assert default.name == 'foo'

    def test_validate_key(self):
        assert FilterSet().validate_key('foo') is None
        assert FilterSet().validate_key('foo__bar') is None
        assert FilterSet().validate_key('foo__bar!') is None

        with pytest.raises(forms.ValidationError):
            FilterSet().validate_key('f!oo')

    def test_get_filter_backend(self):
        backend = FilterSet().get_filter_backend()

        assert isinstance(backend, DjangoFilterBackend)

    def test_filter_no_queryset(self):
        fs = FilterSet()
        with pytest.raises(AssertionError):
            fs.filter()

    def test_filter_data_not_querydict(self):
        fs = FilterSet(queryset=[])
        with pytest.raises(AssertionError):
            fs.filter()

    def test_get_specs(self):
        class BarFilterSet(FilterSet):
            other = Filter(source='stuff',
                           form_field=forms.CharField(),
                           default_lookup='contains')
            thing = Filter(form_field=forms.IntegerField(min_value=0, max_value=15))

        class FooFilterSet(FilterSet):
            field = Filter(form_field=forms.CharField())
            bar = BarFilterSet()

        def _test(data, expected, **kwargs):
            fs = FooFilterSet(
                data=QueryDict(data),
                queryset=[],
                **kwargs
            )

            assert set(fs.get_specs()) == set(expected)

        _test('field=earth&bar__other=mars', [
            FilterSpec(['field'], 'exact', 'earth', False),
            FilterSpec(['bar', 'stuff'], 'contains', 'mars', False),
        ])
        _test('field!=earth&bar__other=mars', [
            FilterSpec(['field'], 'exact', 'earth', True),
            FilterSpec(['bar', 'stuff'], 'contains', 'mars', False),
        ])
        _test('field__in=earth,pluto&bar__other__icontains!=mars', [
            FilterSpec(['field'], 'in', ['earth', 'pluto'], False),
            FilterSpec(['bar', 'stuff'], 'icontains', 'mars', True),
        ])
        _test('fields__in=earth,pluto&bar__other__icontains!=mars', [
            FilterSpec(['bar', 'stuff'], 'icontains', 'mars', True),
        ])
        _test('field__in=earth,pluto&bar__ot!her__icontains!=mars', [
            FilterSpec(['field'], 'in', ['earth', 'pluto'], False),
        ])
        _test('bar__thing=5', [
            FilterSpec(['bar', 'thing'], 'exact', 5, False),
        ])
        _test('bar__thing__in=5,10,15', [
            FilterSpec(['bar', 'thing'], 'in', [5, 10, 15], False),
        ])
        _test('bar__thing__range=5,10', [
            FilterSpec(['bar', 'thing'], 'range', [5, 10], False),
        ])
        _test('bar=5', [])
        _test('bar__thing__range=5,10,15', [], strict_mode=StrictMode.drop)
        _test('bar__thing__range=5,100', [], strict_mode=StrictMode.drop)
        _test('bar__thing=100', [], strict_mode=StrictMode.drop)
        _test('bar__thing__in=100,50', [], strict_mode=StrictMode.drop)
        _test('bar__thing__in=100,5', [
            FilterSpec(['bar', 'thing'], 'in', [5], False)
        ], strict_mode=StrictMode.drop)
        _test('bar__thing__in=100,5', [
            FilterSpec(['bar', 'thing'], 'in', [5], False)
        ], strict_mode=StrictMode.empty)

        with pytest.raises(forms.ValidationError):
            _test('bar__thing__in=100,5', [], strict_mode=StrictMode.fail)

        with pytest.raises(Empty):
            _test('bar__thing__in=100,50', [], strict_mode=StrictMode.empty)

        with pytest.raises(Empty):
            _test('bar__thing__range=5,100', [], strict_mode=StrictMode.empty)

    def test_get_specs_using_default_filter(self):
        class BarFilterSet(FilterSet):
            id = Filter(form_field=forms.IntegerField(),
                        is_default=True)
            other = Filter(source='stuff',
                           form_field=forms.CharField(),
                           default_lookup='contains')
            thing = Filter(form_field=forms.IntegerField(min_value=0, max_value=15))

        class FooFilterSet(FilterSet):
            field = Filter(form_field=forms.CharField(), is_default=True)
            bar = BarFilterSet()

        def _test(data, expected, **kwargs):
            fs = FooFilterSet(
                data=QueryDict(data),
                queryset=[],
                **kwargs
            )

            assert set(fs.get_specs()) == set(expected)

        _test('bar=5', [
            FilterSpec(['bar', 'id'], 'exact', 5, False),
        ])
        _test('bar__isnull=True', [
            FilterSpec(['bar', 'id'], 'isnull', True, False),
        ])
        _test('bar__gt=foo', [], strict_mode=StrictMode.drop)
        _test('page=1', [], strict_mode=StrictMode.fail)

        with pytest.raises(forms.ValidationError):
            _test('bar=aa', [], strict_mode=StrictMode.fail)

        with pytest.raises(Empty):
            _test('bar__in=aa', [], strict_mode=StrictMode.empty)

    def test_filter_one_to_one(self, one_to_one):
        class PlaceFilterSet(FilterSet):
            pk = Filter(form_field=forms.IntegerField(min_value=0), is_default=True)
            name = Filter(form_field=forms.CharField(max_length=50))
            address = Filter(form_field=forms.CharField(max_length=80))

        class RestaurantFilterSet(FilterSet):
            pk = Filter(form_field=forms.IntegerField(min_value=0), is_default=True)
            place = PlaceFilterSet()
            place_id = Filter(form_field=forms.IntegerField(min_value=0))
            serves_hot_dogs = Filter(form_field=forms.BooleanField(required=False))
            serves_pizza = Filter(form_field=forms.BooleanField(required=False))

        class WaiterFilterSet(FilterSet):
            pk = Filter(form_field=forms.IntegerField(min_value=0), is_default=True)
            restaurant = RestaurantFilterSet()
            name = Filter(form_field=forms.CharField(max_length=50))

        def _test(fs, data, qs, expected, count):
            _fs = fs(
                data=QueryDict(data),
                queryset=qs,
            )

            filtered = _fs.filter()
            assert filtered.count() == count
            assert set(filtered) == set(expected)

        _test(
            RestaurantFilterSet,
            'pk=hello',
            Restaurant.objects.all(),
            Restaurant.objects.none(),
            0
        )
        _test(
            RestaurantFilterSet,
            'place__name__startswith=Demon',
            Restaurant.objects.all(),
            Restaurant.objects.filter(place__name__startswith='Demon'),
            1
        )
        _test(
            RestaurantFilterSet,
            'place__address__contains!=Ashland',
            Restaurant.objects.all(),
            Restaurant.objects.exclude(place__address__contains='Ashland'),
            1
        )
        _test(
            RestaurantFilterSet,
            'place_id__isnull=True',
            Restaurant.objects.all(),
            Restaurant.objects.filter(place_id__isnull=True),
            0
        )
        _test(
            RestaurantFilterSet,
            'place_id__isnull=False',
            Restaurant.objects.all(),
            Restaurant.objects.filter(place_id__isnull=False),
            2
        )
        _test(
            WaiterFilterSet,
            'restaurant__place__pk=1',
            Waiter.objects.all(),
            Waiter.objects.filter(restaurant__place=1),
            2
        )
        _test(
            WaiterFilterSet,
            'restaurant__place=1',
            Waiter.objects.all(),
            Waiter.objects.filter(restaurant__place=1),
            2
        )
        _test(
            WaiterFilterSet,
            'restaurant__place__name__startswith=Demon',
            Waiter.objects.all(),
            Waiter.objects.filter(restaurant__place__name__startswith="Demon"),
            2
        )
        _test(
            WaiterFilterSet,
            ('restaurant__place__name__startswith=Demon'
             '&name__icontains!=jon'),
            Waiter.objects.all(),
            (Waiter.objects
             .filter(restaurant__place__name__startswith="Demon")
             .exclude(name__icontains='jon')),
            1
        )



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

from url_filter.utils import FilterSpec, LookupConfig, SubClassDict, dictify


class TestFilterSpec(object):
    def test_repr(self):
        assert repr(FilterSpec(['a', 'b'], 'exact', 'value', False)) == (
            '<FilterSpec a.b exact {}>'.format(repr('value'))
        )
        assert repr(FilterSpec(['a', 'b'], 'exact', 'value', True)) == (
            '<FilterSpec a.b NOT exact {}>'.format(repr('value'))
        )

        class Foo(object):
            def foo(self):
                pass

        f = Foo()

        assert repr(FilterSpec(['a', 'b'], 'exact', 'value', False, filter_callable=f.foo)) == (
            '<FilterSpec a.b exact {} via Foo.foo>'.format(repr('value'))
        )

    def test_equality(self):
        a = FilterSpec(['a', 'b'], 'exact', 'value', False)
        b = FilterSpec(['a', 'b'], 'exact', 'value', False)
        c = FilterSpec(['a', 'b'], 'exact', 'value', True)
        d = FilterSpec(['a', 'b'], 'contains', 'value', False)

        assert a == b
        assert a != c
        assert a != d


class TestLookupConfig(object):
    def test_repr(self):
        data = {
            'key': 'value',
        }
        config = LookupConfig('foo', data)

        assert repr(config) == (
            '<LookupConfig foo=>{}>'.format(repr({'key': 'value'}))
        )

    def test_properties(self):
        data = {
            'key': 'value',
        }
        config = LookupConfig('foo', data)

        assert config.name == 'key'
        assert config.value.data == 'value'
        assert config.is_key_value()

    def test_as_dict(self):
        data = {
            'one': {
                'two': {
                    'three': 'value',
                }
            }
        }

        config = LookupConfig('key', data)

        assert config.as_dict() == data


class TestSubClassDict(object):
    def test_get(self):
        class Klass(object):
            pass

        class Foo(object):
            pass

        class Bar(Foo):
            pass

        mapping = SubClassDict({
            'a': 'b',
            'z': 'b',
            Foo: 'foo',
            Klass: 'klass',
            'key': 'value',
        })

        assert mapping.get('a') == 'b'
        assert mapping.get('key') == 'value'
        assert mapping.get(Klass) == 'klass'
        assert mapping.get(Foo) == 'foo'
        assert mapping.get(Bar) == 'foo'
        assert mapping.get('not-there') is None


def test_dictify():
    a = {'data': 'here'}
    assert dictify(a) is a

    class Foo(object):
        def __init__(self):
            self.a = 'a'
            self.b = 'b'
            self._c = 'c'

    assert dictify(Foo()) == {
        'a': 'a',
        'b': 'b',
    }

    class Bar(object):
        __slots__ = ['a', 'b', '_c']

        def __init__(self):
            self.a = 'a'
            self.b = 'b'
            self._c = 'c'

    assert dictify(Bar()) == {
        'a': 'a',
        'b': 'b',
    }



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from datetime import date, datetime, time
from decimal import Decimal

import six
from django import forms

from ..backends.plain import PlainFilterBackend
from ..exceptions import SkipFilter
from ..filters import Filter
from ..utils import SubClassDict, dictify
from .base import BaseModelFilterSet


DATA_TYPES_MAPPING = SubClassDict({
    six.string_types: forms.CharField(),
    six.integer_types: forms.IntegerField(),
    bool: forms.BooleanField(required=False),
    float: forms.FloatField(),
    Decimal: forms.DecimalField(),
    datetime: forms.DateTimeField(),
    date: forms.DateField(),
    time: forms.TimeField(),
})


class PlainModelFilterSet(BaseModelFilterSet):
    """
    :class:`.FilterSet` for plain Python objects.

    The filterset can be configured via ``Meta`` class attribute,
    very much like Django's ``ModelForm`` is configured.
    """
    filter_backend_class = PlainFilterBackend

    def _build_state(self):
        return dictify(self.Meta.model)

    def _build_filter(self, name, model):
        value = model.get(self._get_filter_extra_kwargs(name).get('source', name))
        primitive = DATA_TYPES_MAPPING.get(type(value))

        if primitive:
            return self._build_filter_from_field(name, value)

        elif isinstance(value, (list, tuple, set)) and value:
            value = list(value)[0]
            if DATA_TYPES_MAPPING.get(type(value)):
                return self._build_filter_from_field(name, value)
            if not isinstance(value, dict):
                raise SkipFilter
            if not self.Meta.allow_related:
                raise SkipFilter
            return self._build_filterset_from_related_field(name, value)

        elif isinstance(value, dict):
            if not self.Meta.allow_related:
                raise SkipFilter
            return self._build_filterset_from_related_field(name, value)

    def _get_model_field_names(self):
        return list(dictify(self.Meta.model).keys())

    def _build_filter_from_field(self, name, field):
        return Filter(
            form_field=DATA_TYPES_MAPPING.get(type(field)),
            **self._get_filter_extra_kwargs(name)
        )

    def _build_filterset_from_related_field(self, name, field):
        if not field:
            raise SkipFilter
        return self._build_filterset(
            name.title(),
            name,
            {'model': field},
            PlainModelFilterSet,
        )



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

from django import forms

from url_filter.filters import Filter
from url_filter.filtersets.plain import PlainModelFilterSet


class TestPlainModelFilterSet(object):
    def test_get_filters_no_relations_place(self):
        class PlaceFilterSet(PlainModelFilterSet):
            class Meta(object):
                model = {
                    "id": 1,
                    "restaurant": {
                        "place": 1,
                        "waiters": [
                            {
                                "id": 1,
                                "name": "Joe",
                                "restaurant": 1
                            },
                            {
                                "id": 2,
                                "name": "Jonny",
                                "restaurant": 1
                            }
                        ],
                        "serves_hot_dogs": True,
                        "serves_pizza": False,
                    },
                    "nicknames": ["dogs"],
                    "name": "Demon Dogs",
                    "address": "944 W. Fullerton",
                    "ignored": [{}],
                }
                allow_related = False
                extra_kwargs = {
                    'id': {'no_lookup': True},
                }

        filters = PlaceFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'name', 'address', 'nicknames',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert filters['id'].no_lookup is True
        assert isinstance(filters['name'], Filter)
        assert isinstance(filters['name'].form_field, forms.CharField)
        assert isinstance(filters['address'], Filter)
        assert isinstance(filters['address'].form_field, forms.CharField)
        assert isinstance(filters['nicknames'], Filter)
        assert isinstance(filters['nicknames'].form_field, forms.CharField)

    def test_get_filters_no_relations_place_diff_source(self):
        class PlaceFilterSet(PlainModelFilterSet):
            class Meta(object):
                model = {
                    "id": 1,
                    "restaurant": {
                        "place": 1,
                        "waiters": [
                            {
                                "id": 1,
                                "name": "Joe",
                                "restaurant": 1
                            },
                            {
                                "id": 2,
                                "name": "Jonny",
                                "restaurant": 1
                            }
                        ],
                        "serves_hot_dogs": True,
                        "serves_pizza": False,
                    },
                    "nicknames": ["dogs"],
                    "name": "Demon Dogs",
                    "address": "944 W. Fullerton",
                    "ignored": [{}],
                }
                allow_related = False
                fields = ['id', 'name', 'location', 'nicknames']
                extra_kwargs = {
                    'id': {'no_lookup': True},
                    'location': {'source': 'address'}
                }

        filters = PlaceFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'name', 'location', 'nicknames',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert filters['id'].no_lookup is True
        assert isinstance(filters['name'], Filter)
        assert isinstance(filters['name'].form_field, forms.CharField)
        assert isinstance(filters['location'], Filter)
        assert isinstance(filters['location'].form_field, forms.CharField)
        assert isinstance(filters['nicknames'], Filter)
        assert isinstance(filters['nicknames'].form_field, forms.CharField)

    def test_get_filters_with_both_reverse_and_direct_relations(self):
        class PlaceFilterSet(PlainModelFilterSet):
            class Meta(object):
                model = {
                    "id": 1,
                    "restaurant": {
                        "place": 1,
                        "waiters": [
                            {
                                "id": 1,
                                "name": "Joe",
                                "restaurant": 1
                            },
                            {
                                "id": 2,
                                "name": "Jonny",
                                "restaurant": 1
                            },
                        ],
                        "serves_hot_dogs": True,
                        "serves_pizza": False,
                        "dummy": [{}],
                        "ignored": [object()],
                    },
                    "nicknames": ["dogs"],
                    "name": "Demon Dogs",
                    "address": "944 W. Fullerton"
                }
                extra_kwargs = {
                    'restaurant': {
                        'place': {'no_lookup': True},
                    },
                }

        filters = PlaceFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'address', 'name', 'nicknames', 'restaurant',
        }
        assert set(filters['restaurant'].filters.keys()) == {
            'place', 'serves_hot_dogs', 'serves_pizza', 'waiters',
        }
        assert set(filters['restaurant'].filters['waiters'].filters.keys()) == {
            'id', 'name', 'restaurant',
        }

        assert filters['restaurant'].filters['place'].no_lookup is True

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert isinstance(filters['address'], Filter)
        assert isinstance(filters['address'].form_field, forms.CharField)
        assert isinstance(filters['name'], Filter)
        assert isinstance(filters['name'].form_field, forms.CharField)
        assert isinstance(filters['restaurant'], PlainModelFilterSet)
        assert isinstance(filters['restaurant'].filters['waiters'], PlainModelFilterSet)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from django.core.validators import (
    MaxLengthValidator as _MaxLengthValidator,
    MinLengthValidator as _MinLengthValidator,
)
from django.utils.deconstruct import deconstructible
from django.utils.translation import ungettext_lazy


@deconstructible
class MinLengthValidator(_MinLengthValidator):
    """
    Customer Django min length validator with better-suited error message
    """
    code = 'min_length'
    message = ungettext_lazy(
        'Ensure this value has at least %(limit_value)d items (it has %(show_value)d).',
        'Ensure this value has at least %(limit_value)d items (it has %(show_value)d).',
        'limit_value'
    )

    def compare(self, a, b):
        return a < b

    def clean(self, x):
        return len(x)


@deconstructible
class MaxLengthValidator(_MaxLengthValidator):
    """
    Customer Django max length validator with better-suited error message
    """
    code = 'max_length'
    message = ungettext_lazy(
        'Ensure this value has at most %(limit_value)d items (it has %(show_value)d).',
        'Ensure this value has at most %(limit_value)d items (it has %(show_value)d).',
        'limit_value'
    )

    def compare(self, a, b):
        return a > b

    def clean(self, x):
        return len(x)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import coreapi
import coreschema
from django import forms
from django.db.models.constants import LOOKUP_SEP

from ..fields import MultipleValuesField
from ..filtersets import FilterSet
from .drf import URLFilterBackend


DESCRIPTION = {
    'contains': 'Match when string contains given substring',
    'day': 'Match by day of the month',
    'endswith': 'Match when string ends with given substring',
    'exact': 'Match exactly the value as is',
    'gt': 'Match when value is greater then given value',
    'gte': 'Match when value is greater or equal then given value',
    'hour': 'Match by the hour (24 hour) value of the timestamp',
    'icontains': 'Case insensitive match when string contains given substring',
    'iendswith': 'Case insensitive match when string ends with given substring',
    'iexact': 'Case insensitive match exactly the value as is',
    'iin': 'Case insensitive match when value is any of given comma separated values',
    'in': 'Match when value is any of given comma separated values',
    'iregex': 'Case insensitive match when value is within comma separated range',
    'isnull': 'Match when value is NULL',
    'istartswith': 'Case insensitive match when string starts with given substring',
    'lt': 'Match when value is less then given value',
    'lte': 'Match when value is less or equal then given value',
    'minute': 'Match by the minute value of the timestamp',
    'month': 'Match by the month value of the timestamp',
    'range': 'Match when value is within comma separated range',
    'regex': 'Match string by regex pattern',
    'second': 'Match by the second value of the timestamp',
    'startswith': 'Match when string starts with given substring',
    'week_day': 'Match by week day (1-Sunday to 7-Saturday) of the timestamp',
    'year': 'Match by the year value of the timestamp',
}

FORM_FIELD_TO_SCHEMA = {
    MultipleValuesField: coreschema.Array,
    forms.BooleanField: coreschema.Boolean,
    forms.IntegerField: coreschema.Integer,
    forms.FloatField: coreschema.Number,
}


def _field_to_schema(field, lookup):
    form_field = field.get_form_field(lookup)
    return FORM_FIELD_TO_SCHEMA.get(type(form_field), coreschema.String)(description=DESCRIPTION.get(lookup))


def _all_filters(filterset, prefix=()):
    for name, field in filterset.filters.items():
        if isinstance(field, FilterSet):
            for i in _all_filters(field, prefix=prefix + (name,)):
                yield i

        else:
            yield coreapi.Field(
                name=LOOKUP_SEP.join(prefix + (name,)),
                required=False,
                location="query",
                schema=_field_to_schema(field, field.default_lookup),
            )
            if field.no_lookup:
                continue
            for lookup in field.lookups:
                yield coreapi.Field(
                    name=LOOKUP_SEP.join(prefix + (name, lookup)),
                    required=False,
                    location="query",
                    schema=_field_to_schema(field, lookup),
                )


class CoreAPIURLFilterBackend(URLFilterBackend):
    """
    Same as :py:class:`url_filter.integrations.drf.DjangoFilterBackend` except
    this backend also implements coreapi interface for autogenerated API docs.
    """

    def get_schema_fields(self, view):
        """
        Get coreapi filter definitions

        Returns all filters including their supported lookups.
        """
        queryset = view.get_queryset()
        filter_class = self.get_filter_class(view, queryset)

        return (
            []
            if not filter_class
            else list(_all_filters(filter_class(data={}, queryset=queryset)))
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import pytest
import six
from alchemy_mock.comparison import ExpressionMatcher
from sqlalchemy import func
from sqlalchemy.orm import joinedload
from sqlalchemy.types import String

from test_project.one_to_one.alchemy import Place, Restaurant, Waiter
from url_filter.backends.sqlalchemy import SQLAlchemyFilterBackend
from url_filter.utils import FilterSpec


def assert_alchemy_expressions_equal(exp1, exp2):
    assert ExpressionMatcher(exp1) == exp2


class TestSQLAlchemyFilterBackend(object):
    def test_init(self, alchemy_db):
        backend = SQLAlchemyFilterBackend(
            alchemy_db.query(Place),
            context={'context': 'here'},
        )

        assert backend.model is Place
        assert backend.context == {'context': 'here'}

        with pytest.raises(AssertionError):
            SQLAlchemyFilterBackend(
                alchemy_db.query(Place, Restaurant),
            )

    def test_empty(self, alchemy_db):
        backend = SQLAlchemyFilterBackend(
            alchemy_db.query(Place),
            context={'context': 'here'},
        )

        assert 'WHERE 0 = 1' in six.text_type(backend.empty())

    def test_get_model(self, alchemy_db):
        backend = SQLAlchemyFilterBackend(alchemy_db.query(Place))

        assert backend.get_model() is Place

    def test_filter_no_specs(self, alchemy_db):
        qs = alchemy_db.query(Place)

        backend = SQLAlchemyFilterBackend(qs)
        backend.bind([])

        assert backend.filter() is qs

    def test_filter(self, alchemy_db):
        backend = SQLAlchemyFilterBackend(
            alchemy_db.query(Place),
        )
        backend.bind([
            FilterSpec(['restaurant', 'waiter_set', 'name'], 'exact', 'John', False),
        ])

        filtered = backend.filter()

        sql = six.text_type(filtered)
        assert sql == (
            'SELECT one_to_one_place.id AS one_to_one_place_id, '
            'one_to_one_place.name AS one_to_one_place_name, '
            'one_to_one_place.address AS one_to_one_place_address \n'
            'FROM one_to_one_place '
            'JOIN one_to_one_restaurant '
            'ON one_to_one_restaurant.place_id = one_to_one_place.id '
            'JOIN one_to_one_waiter '
            'ON one_to_one_waiter.restaurant_id = one_to_one_restaurant.place_id '
            '\nWHERE one_to_one_waiter.name ={}'.format(sql.rsplit('=', 1)[-1])
        )

    def test_filter_already_eagerloaded(self, alchemy_db):
        backend = SQLAlchemyFilterBackend(
            alchemy_db.query(Place).options(joinedload(Place.restaurant).joinedload(Restaurant.waiter_set)),
        )
        backend.bind([
            FilterSpec(['restaurant', 'waiter_set', 'name'], 'exact', 'John', False),
        ])

        filtered = backend.filter()

        sql = six.text_type(filtered)
        assert 'place JOIN one_to_one' not in sql
        assert 'place LEFT OUTER JOIN one_to_one' in sql

    def _test_build_clause(self, alchemy_db, name, lookup, value, expected, is_negated=False):
        backend = SQLAlchemyFilterBackend(
            alchemy_db.query(Place),
        )

        clause, to_join = backend.build_clause(
            FilterSpec(['restaurant', 'waiter_set', name], lookup, value, is_negated)
        )

        assert to_join == [Place.restaurant, Restaurant.waiter_set]
        assert_alchemy_expressions_equal(clause, expected)

    def test_build_clause_contains(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'contains', 'John',
            Waiter.name.contains('John')
        )

    def test_build_clause_endswith(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'endswith', 'John',
            Waiter.name.endswith('John')
        )

    def test_build_clause_exact(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'exact', 'John',
            Waiter.name == 'John'
        )

    def test_build_clause_exact_negated(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'exact', 'John',
            Waiter.name != 'John',
            True
        )

    def test_build_clause_gt(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'id', 'gt', 1,
            Waiter.id > 1
        )

    def test_build_clause_gte(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'id', 'gte', 1,
            Waiter.id >= 1
        )

    def test_build_clause_icontains(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'icontains', 'Django',
            func.lower(Waiter.name).contains('django')
        )

    def test_build_clause_icontains_cant_lower(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'icontains', 5,
            func.lower(Waiter.name).contains(5)
        )

    def test_build_clause_iendswith(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'iendswith', 'Django',
            func.lower(Waiter.name).endswith('django')
        )

    def test_build_clause_iexact(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'iexact', 'Django',
            func.lower(Waiter.name) == 'django'
        )

    def test_build_clause_in(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'in', ['Django', 'rocks'],
            Waiter.name.in_(['Django', 'rocks'])
        )

    def test_build_clause_iin(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'iin', ['Django', 'rocks'],
            func.lower(Waiter.name).in_(['django', 'rocks'])
        )

    def test_build_clause_isnull(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'isnull', True,
            Waiter.name == None  # noqa
        )
        self._test_build_clause(
            alchemy_db, 'name', 'isnull', False,
            Waiter.name != None  # noqa
        )

    def test_build_clause_istartswith(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'istartswith', 'Django',
            func.lower(Waiter.name).startswith('django')
        )

    def test_build_clause_lt(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'id', 'lt', 1,
            Waiter.id < 1
        )

    def test_build_clause_lte(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'id', 'lte', 1,
            Waiter.id <= 1
        )

    def test_build_clause_range(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'id', 'range', [1, 5],
            Waiter.id.between(1, 5)
        )

    def test_build_clause_startswith(self, alchemy_db):
        self._test_build_clause(
            alchemy_db, 'name', 'startswith', 'Django',
            Waiter.name.startswith('Django')
        )

    def test__get_properties_for_model(self):
        properties = SQLAlchemyFilterBackend._get_properties_for_model(Waiter)

        assert set(properties) == {'restaurant', 'id', 'restaurant_id', 'name'}

    def test__get_column_for_field(self):
        properties = SQLAlchemyFilterBackend._get_properties_for_model(Waiter)
        name = properties['name']
        column = SQLAlchemyFilterBackend._get_column_for_field(name)

        assert column.key == 'name'
        assert isinstance(column.type, String)
        assert column.table is Waiter.__table__

    def test__get_attribute_for_field(self):
        properties = SQLAlchemyFilterBackend._get_properties_for_model(Waiter)
        name = properties['name']
        attr = SQLAlchemyFilterBackend._get_attribute_for_field(name)

        assert attr is Waiter.name

    def test__get_related_model_for_field(self):
        properties = SQLAlchemyFilterBackend._get_properties_for_model(Waiter)
        restaurant = properties['restaurant']
        model = SQLAlchemyFilterBackend._get_related_model_for_field(restaurant)

        assert model is Restaurant



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import mock

from test_project.one_to_one.models import Place
from url_filter.backends.django import DjangoFilterBackend
from url_filter.utils import FilterSpec


class TestDjangoFilterBackend(object):
    def test_init(self):
        backend = DjangoFilterBackend(
            Place.objects.all(),
            context={'context': 'here'},
        )

        assert backend.model is Place
        assert backend.context == {'context': 'here'}

    def test_empty(self):
        backend = DjangoFilterBackend(
            Place.objects.all(),
            context={'context': 'here'},
        )

        assert backend.empty().count() == 0

    def test_get_model(self):
        backend = DjangoFilterBackend(Place.objects.all())

        assert backend.get_model() is Place

    def test_bind(self):
        backend = DjangoFilterBackend(Place.objects.all())

        assert backend.specs == []
        backend.bind([1, 2])
        assert backend.specs == [1, 2]

    def test_includes(self):
        backend = DjangoFilterBackend(Place.objects.all())
        backend.bind([
            FilterSpec(['name'], 'exact', 'value', False),
            FilterSpec(['address'], 'contains', 'value', True),
        ])

        assert list(backend.includes) == [
            FilterSpec(['name'], 'exact', 'value', False),
        ]

    def test_excludes(self):
        backend = DjangoFilterBackend(Place.objects.all())
        backend.bind([
            FilterSpec(['name'], 'exact', 'value', False),
            FilterSpec(['address'], 'contains', 'value', True),
        ])

        assert list(backend.excludes) == [
            FilterSpec(['address'], 'contains', 'value', True),
        ]

    def test_prepare_spec(self):
        backend = DjangoFilterBackend(Place.objects.all())
        spec = backend._prepare_spec(FilterSpec(['name'], 'exact', 'value'))

        assert spec == 'name__exact'

    def test_filter(self):
        qs = mock.Mock()

        backend = DjangoFilterBackend(qs)
        backend.model = Place
        backend.bind([
            FilterSpec(['name'], 'exact', 'value', False),
            FilterSpec(['address'], 'contains', 'value', True),
        ])

        result = backend.filter()

        assert result == qs.filter.return_value.exclude.return_value
        qs.filter.assert_called_once_with(name__exact='value')
        qs.filter.return_value.exclude.assert_called_once_with(address__contains='value')

    def test_filter_to_many(self):
        qs = mock.Mock()

        backend = DjangoFilterBackend(qs)
        backend.model = Place
        backend.bind([
            FilterSpec(['restaurant', 'waiter', 'name'], 'exact', 'value', False),
        ])

        result = backend.filter()

        assert result == qs.filter.return_value.distinct.return_value
        qs.filter.assert_called_once_with(restaurant__waiter__name__exact='value')

    def test_filter_callable_specs(self):
        qs = mock.Mock()

        def foo(queryset, spec):
            return queryset.filter(spec)

        spec = FilterSpec(['name'], 'exact', 'value', False, foo)
        backend = DjangoFilterBackend(qs)
        backend.bind([spec])

        result = backend.filter()

        assert result == qs.filter.return_value
        qs.filter.assert_called_once_with(spec)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals


class SkipFilter(Exception):
    """
    Exception to be used when any particular filter
    within the :class:`url_filter.filtersets.base.FilterSet` should be skipped.

    Possible reasons for skipping the field:

    * filter lookup config is invalid
      (e.g. using wrong field name -
      field is not present in filter set)
    * filter lookup value is invalid
      (e.g. submitted "a" for integer field)
    """


class Empty(Exception):
    """
    Exception to be used when filter backend should return
    empty queryset when any of filter validations has failed.
    """



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from django import forms

from .validators import MaxLengthValidator, MinLengthValidator


class MultipleValuesField(forms.CharField):
    """
    Custom Django field for validating/cleaning multiple
    values given in a single value separated by a delimiter.

    Parameters
    ----------
    child : Field, optional
        Another Django form field which should be used.
    min_values : int, optional
        Minimum number of values which must be provided.
        By default at least 2 values are required.
    max_values : int, optional
        Maximum number of values which can be provided.
        By default no maximum is enforced.
    max_validators : list, optional
        Additional validators which should be used to validate
        all values once split by the delimiter.
    delimiter : str, optional
        The delimiter by which the value will be split into
        multiple values.
        By default ``,`` is used.
    all_valid : bool, optional
        When ``False``, if any specific item does not pass validation
        it is ignored without failing complete field validation.
        Default is ``True`` which enforces validation for all items.
    """
    def __init__(self, child=None,
                 min_values=2, max_values=None,
                 many_validators=None,
                 delimiter=',',
                 all_valid=True,
                 *args, **kwargs):
        self.child = child or forms.CharField()
        self.delimiter = delimiter
        self.all_valid = all_valid

        super(MultipleValuesField, self).__init__(*args, **kwargs)

        self.many_validators = many_validators or []
        if min_values:
            self.many_validators.append(MinLengthValidator(min_values))
        if max_values:
            self.many_validators.append(MaxLengthValidator(max_values))

    def clean(self, value):
        """
        Custom ``clean`` which first validates the value first by using
        standard ``CharField`` and if all passes, it applies
        similar validations for each value once its split.
        """
        value = self.to_python(value)
        self.validate(value)
        self.run_validators(value)

        if not value:
            return

        values = self.many_to_python(value)
        self.many_validate(values)
        self.many_run_validators(values)

        return values

    def many_to_python(self, value):
        """
        Method responsible to split the value into multiple
        values by using the delimiter and cleaning each one
        as per the child field.
        """
        values = []
        for i in value.split(self.delimiter):
            try:
                values.append(self.child.clean(i))
            except forms.ValidationError:
                if self.all_valid:
                    raise
        return values

    def many_validate(self, values):
        """
        Hook for validating all values.
        """
        if not values and self.required:
            raise forms.ValidationError(self.error_messages['required'], code='required')

    def many_run_validators(self, values):
        """
        Run each validation from ``many_validators`` for the cleaned values.
        """
        if not values:
            return

        errors = []
        for v in self.many_validators:
            try:
                v(values)
            except forms.ValidationError as e:
                if hasattr(e, 'code') and e.code in self.error_messages:
                    e = forms.ValidationError(self.error_messages[e.code], e.code, e.params)
                errors.extend(e.error_list)
        if errors:
            raise forms.ValidationError(errors)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from datetime import datetime

from url_filter.backends.plain import PlainFilterBackend
from url_filter.utils import FilterSpec


class Bunch(object):
    def __init__(self, **kwargs):
        self.__dict__ = kwargs


DATA = [
    {
        "id": 1,
        "restaurant": {
            "place": 1,
            "waiters": [
                {
                    "id": 1,
                    "name": "Joe",
                    "restaurant": 1
                },
                {
                    "id": 2,
                    "name": "Jonny",
                    "restaurant": 1
                }
            ],
            "serves_hot_dogs": True,
            "serves_pizza": False
        },
        "name": "Demon Dogs",
        "address": "944 W. Fullerton",
        "nicknames": [
            "dogs",
        ],
        "created": datetime(2015, 6, 12, 9, 30, 55),
    },
    {
        "id": 2,
        "restaurant": {
            "place": 2,
            "waiters": [
                Bunch(**{
                    "id": 3,
                    "name": "Steve",
                    "restaurant": 2
                })
            ],
            "serves_hot_dogs": True,
            "serves_pizza": False
        },
        "name": "Ace Hardware",
        "address": "1013 N. Ashland",
        "nicknames": [
            "Ace",
            "ace",
        ],
        "created": datetime(2014, 5, 12, 14, 30, 37),
        "nulldata": None,
    }
]


class TestPlainFilterBackend(object):
    def test_get_model(self):
        backend = PlainFilterBackend([])

        assert backend.get_model() is object
        assert not backend.enforce_same_models

    def test_empty(self):
        backend = PlainFilterBackend([])

        assert backend.empty() == []

    def test_filter_no_specs(self):
        qs = ['hello']
        backend = PlainFilterBackend(qs)
        backend.bind([])

        assert backend.filter() is qs

    def _test_filter(self, spec, expected):
        backend = PlainFilterBackend(DATA)
        backend.bind([spec])

        assert backend.filter() == expected

    def test_filter_contains(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'contains', 'Jo', False),
            [DATA[0]]
        )

    def test_filter_endswith(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'endswith', 'e', False),
            DATA
        )

    def test_filter_exact(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'exact', 'John', False),
            []
        )

    def test_filter_exact_simple_list(self):
        self._test_filter(
            FilterSpec(['nicknames'], 'exact', 'ace', False),
            [DATA[1]]
        )

    def test_filter_exact_negated(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'exact', 'John', True),
            DATA
        )

    def test_filter_gt(self):
        self._test_filter(
            FilterSpec(['id'], 'gt', 1, False),
            [DATA[1]]
        )

    def test_filter_gte(self):
        self._test_filter(
            FilterSpec(['id'], 'gte', 1, False),
            DATA
        )

    def test_filter_icontains(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'icontains', 'jo', False),
            [DATA[0]]
        )

    def test_filter_iendswith(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'iendswith', 'E', False),
            DATA
        )

    def test_filter_iexact(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'iexact', 'joe', False),
            [DATA[0]]
        )

    def test_filter_in(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'in', ['John', 'Steve'], False),
            [DATA[1]]
        )

    def test_filter_iin(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'iin', ['john', 'steve'], False),
            [DATA[1]]
        )

    def test_filter_in_simple_list(self):
        self._test_filter(
            FilterSpec(['nicknames'], 'in', ['ace', 'dogs'], False),
            DATA
        )

    def test_filter_isnull(self):
        self._test_filter(
            FilterSpec(['nulldata'], 'isnull', True, False),
            [DATA[1]]
        )
        self._test_filter(
            FilterSpec(['nulldata'], 'isnull', False, False),
            [DATA[0]]
        )

    def test_filter_istartswith(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'istartswith', 'j', False),
            [DATA[0]]
        )

    def test_filter_lt(self):
        self._test_filter(
            FilterSpec(['id'], 'lt', 2, False),
            [DATA[0]]
        )

    def test_filter_lte(self):
        self._test_filter(
            FilterSpec(['id'], 'lte', 2, False),
            DATA
        )

    def test_filter_range(self):
        self._test_filter(
            FilterSpec(['id'], 'range', [1, 2], False),
            DATA
        )

    def test_filter_startswith(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'startswith', 'J', False),
            [DATA[0]]
        )

    def test_filter_regex(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'regex', r'^J.*', False),
            [DATA[0]]
        )

    def test_filter_iregex(self):
        self._test_filter(
            FilterSpec(['restaurant', 'waiters', 'name'], 'iregex', r'^j.*', False),
            [DATA[0]]
        )

    def test_filter_day(self):
        self._test_filter(
            FilterSpec(['created'], 'day', 12, False),
            DATA
        )

    def test_filter_hour(self):
        self._test_filter(
            FilterSpec(['created'], 'hour', 9, False),
            [DATA[0]]
        )

    def test_filter_second(self):
        self._test_filter(
            FilterSpec(['created'], 'second', 37, False),
            [DATA[1]]
        )

    def test_filter_minute(self):
        self._test_filter(
            FilterSpec(['created'], 'minute', 30, False),
            DATA
        )

    def test_filter_month(self):
        self._test_filter(
            FilterSpec(['created'], 'month', 5, False),
            [DATA[1]]
        )

    def test_filter_year(self):
        self._test_filter(
            FilterSpec(['created'], 'year', 2015, False),
            [DATA[0]]
        )

    def test_filter_week_day(self):
        self._test_filter(
            FilterSpec(['created'], 'week_day', 2, False),  # Monday
            [DATA[1]]
        )

    def test_filter_exception_handling(self):
        self._test_filter(
            FilterSpec(['id'], 'week_day', 1, False),
            DATA
        )



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import pytest
from django import forms
from django.db import models

from test_project.generic.models import ModelB
from test_project.many_to_many.models import Article as M2MArticle, Publication
from test_project.many_to_one.models import Article as M2OArticle
from test_project.one_to_one.models import Place, Restaurant
from url_filter.exceptions import SkipFilter
from url_filter.filters import Filter
from url_filter.filtersets import ModelFilterSet


class TestModelFilterSet(object):
    def test_get_filters_no_model(self):
        class PlaceFilterSet(ModelFilterSet):
            pass

        with pytest.raises(AssertionError):
            PlaceFilterSet().get_filters()

    def test_get_filters_no_relations_place(self):
        class PlaceFilterSet(ModelFilterSet):
            class Meta(object):
                model = Place
                allow_related = False
                allow_related_reverse = False
                extra_kwargs = {
                    'id': {'no_lookup': True},
                }

        filters = PlaceFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'name', 'address',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert filters['id'].no_lookup is True
        assert isinstance(filters['name'], Filter)
        assert isinstance(filters['name'].form_field, forms.CharField)
        assert isinstance(filters['address'], Filter)
        assert isinstance(filters['address'].form_field, forms.CharField)

    def test_get_filters_no_relations_place_diff_source(self):
        class PlaceFilterSet(ModelFilterSet):
            class Meta(object):
                model = Place
                allow_related = False
                allow_related_reverse = False
                fields = ['id', 'name', 'location']
                extra_kwargs = {
                    'id': {'no_lookup': True},
                    'location': {'source': 'address'}
                }

        filters = PlaceFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'name', 'location',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert filters['id'].no_lookup is True
        assert isinstance(filters['name'], Filter)
        assert isinstance(filters['name'].form_field, forms.CharField)
        assert isinstance(filters['location'], Filter)
        assert isinstance(filters['location'].form_field, forms.CharField)

    def test_get_filters_no_relations_place_exclude_address(self):
        class PlaceFilterSet(ModelFilterSet):
            class Meta(object):
                model = Place
                exclude = ['address']
                allow_related = False
                allow_related_reverse = False

        filters = PlaceFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'name',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert isinstance(filters['name'], Filter)
        assert isinstance(filters['name'].form_field, forms.CharField)

    def test_get_filters_no_relations_place_address_overwrite(self):
        class PlaceFilterSet(ModelFilterSet):
            address = Filter(forms.IntegerField())

            class Meta(object):
                model = Place
                allow_related = False
                allow_related_reverse = False

        filters = PlaceFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'name', 'address',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert isinstance(filters['name'], Filter)
        assert isinstance(filters['name'].form_field, forms.CharField)
        assert isinstance(filters['address'], Filter)
        assert isinstance(filters['address'].form_field, forms.IntegerField)

    def test_get_filters_no_relations_restaurant(self):
        class RestaurantFilterSet(ModelFilterSet):
            class Meta(object):
                model = Restaurant
                allow_related = False
                allow_related_reverse = False

        filters = RestaurantFilterSet().get_filters()

        assert set(filters.keys()) == {
            'serves_pizza', 'serves_hot_dogs',
        }

        assert isinstance(filters['serves_pizza'], Filter)
        assert isinstance(filters['serves_pizza'].form_field, forms.BooleanField)
        assert isinstance(filters['serves_hot_dogs'], Filter)
        assert isinstance(filters['serves_hot_dogs'].form_field, forms.BooleanField)

    def test_get_filters_with_only_reverse_relations(self):
        class PlaceFilterSet(ModelFilterSet):
            class Meta(object):
                model = Place

        filters = PlaceFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'name', 'address', 'restaurant',
        }
        assert set(filters['restaurant'].filters.keys()) == {
            'serves_pizza', 'serves_hot_dogs', 'waiter',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert isinstance(filters['name'], Filter)
        assert isinstance(filters['name'].form_field, forms.CharField)
        assert isinstance(filters['address'], Filter)
        assert isinstance(filters['address'].form_field, forms.CharField)
        assert isinstance(filters['restaurant'], ModelFilterSet)

    def test_get_filters_with_both_reverse_and_direct_relations(self):
        class RestaurantFilterSet(ModelFilterSet):
            class Meta(object):
                model = Restaurant
                extra_kwargs = {
                    'place': {
                        'id': {'no_lookup': True},
                    },
                }

        filters = RestaurantFilterSet().get_filters()

        assert set(filters.keys()) == {
            'place', 'waiter', 'serves_hot_dogs', 'serves_pizza',
        }
        assert set(filters['place'].filters.keys()) == {
            'id', 'name', 'address',
        }
        assert set(filters['waiter'].filters.keys()) == {
            'id', 'name',
        }

        assert filters['place'].filters['id'].no_lookup is True

        assert isinstance(filters['serves_hot_dogs'], Filter)
        assert isinstance(filters['serves_hot_dogs'].form_field, forms.BooleanField)
        assert isinstance(filters['serves_pizza'], Filter)
        assert isinstance(filters['serves_pizza'].form_field, forms.BooleanField)
        assert isinstance(filters['place'], ModelFilterSet)
        assert isinstance(filters['waiter'], ModelFilterSet)

    def test_get_filters_with_reverse_many_to_many_relations(self):
        class PublicationFilterSet(ModelFilterSet):
            class Meta(object):
                model = Publication

        filters = PublicationFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'title', 'articles',
        }
        assert set(filters['articles'].filters.keys()) == {
            'id', 'headline',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert isinstance(filters['title'], Filter)
        assert isinstance(filters['title'].form_field, forms.CharField)
        assert isinstance(filters['articles'], ModelFilterSet)

    def test_get_filters_with_many_to_many_relations(self):
        class ArticleFilterSet(ModelFilterSet):
            class Meta(object):
                model = M2MArticle

        filters = ArticleFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'headline', 'publications',
        }
        assert set(filters['publications'].filters.keys()) == {
            'id', 'title',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert isinstance(filters['headline'], Filter)
        assert isinstance(filters['headline'].form_field, forms.CharField)
        assert isinstance(filters['publications'], ModelFilterSet)

    def test_get_filters_with_many_to_one_relations(self):
        class ArticleFilterSet(ModelFilterSet):
            class Meta(object):
                model = M2OArticle

        filters = ArticleFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'headline', 'pub_date', 'reporter',
        }
        assert set(filters['reporter'].filters.keys()) == {
            'id', 'email', 'first_name', 'last_name',
        }

        assert isinstance(filters['id'], Filter)
        assert isinstance(filters['id'].form_field, forms.IntegerField)
        assert isinstance(filters['headline'], Filter)
        assert isinstance(filters['headline'].form_field, forms.CharField)
        assert isinstance(filters['pub_date'], Filter)
        assert isinstance(filters['pub_date'].form_field, forms.DateField)
        assert isinstance(filters['reporter'], ModelFilterSet)

    def test_get_filters_without_generic_foreign_key(self):
        class ModelBFilterSet(ModelFilterSet):
            class Meta(object):
                model = ModelB

        filters = ModelBFilterSet().get_filters()

        assert set(filters.keys()) == {
            'id', 'name', 'a', 'content_type', 'object_id',
        }

    def test_get_form_field_for_field(self):
        fs = ModelFilterSet()

        assert isinstance(fs._get_form_field_for_field(models.CharField()), forms.CharField)
        assert isinstance(fs._get_form_field_for_field(models.AutoField()), forms.IntegerField)
        assert isinstance(fs._get_form_field_for_field(models.FileField()), forms.CharField)

        class TestField(models.Field):
            def formfield(self, **kwargs):
                return

        with pytest.raises(SkipFilter):
            fs._get_form_field_for_field(TestField())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
import os

from setuptools import find_packages, setup

from url_filter import __author__, __version__


def read(fname):
    return (open(os.path.join(os.path.dirname(__file__), fname), 'rb')
            .read().decode('utf-8'))


authors = read('AUTHORS.rst')
history = read('HISTORY.rst').replace('.. :changelog:', '')
licence = read('LICENSE.rst')
readme = read('README.rst')

req = read('requirements.txt').splitlines()
dev_req = read('requirements-dev.txt').splitlines()[1:]

requirements = req + ['setuptools']
test_requirements = req + dev_req

setup(
    name='django-url-filter',
    version=__version__,
    author=__author__,
    description='Django URL Filter provides a safe way to filter data via human-friendly URLs.',
    long_description='\n\n'.join([readme, history, authors, licence]),
    url='https://github.com/miki725/django-url-filter',
    license='MIT',
    packages=find_packages(exclude=['test_project*', 'tests*']),
    install_requires=requirements,
    test_suite='tests',
    tests_require=test_requirements,
    keywords=' '.join([
        'django django-rest-framework',
    ]),
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Development Status :: 2 - Pre-Alpha',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import operator

from django import forms
from django.conf import settings
from django.db import models
from django.db.models.fields.related import ForeignObjectRel, RelatedField

from ..exceptions import SkipFilter
from ..filters import Filter
from ..utils import SubClassDict
from .base import BaseModelFilterSet, ModelFilterSetOptions


__all__ = ['ModelFilterSet', 'DjangoModelFilterSetOptions']


GenericForeignKey = None


if 'django.contrib.contenttypes' in settings.INSTALLED_APPS:
    from django.contrib.contenttypes.fields import GenericForeignKey


MODEL_FIELD_OVERWRITES = SubClassDict({
    models.AutoField: forms.IntegerField(min_value=0),
    models.FileField: lambda m: forms.CharField(max_length=m.max_length),
})


class DjangoModelFilterSetOptions(ModelFilterSetOptions):
    """
    Custom options for ``FilterSet``s used for Django models.

    Attributes
    ----------
    allow_related_reverse : bool, optional
        Flag specifying whether reverse relationships should
        be allowed while creating filter sets for children models.
    """
    def __init__(self, options=None):
        super(DjangoModelFilterSetOptions, self).__init__(options)
        self.allow_related_reverse = getattr(options, 'allow_related_reverse', True)


class ModelFilterSet(BaseModelFilterSet):
    """
    :class:`.FilterSet` for Django models.

    The filterset can be configured via ``Meta`` class attribute,
    very much like Django's ``ModelForm`` is configured.
    """
    filter_options_class = DjangoModelFilterSetOptions

    def _get_model_field_names(self):
        """
        Get a list of all model fields.

        This is used when ``Meta.fields`` is ``None``
        in which case this method returns all model fields.
        """
        return list(map(
            operator.attrgetter('name'),
            self.Meta.model._meta.get_fields()
        ))

    def _get_form_field_for_field(self, field):
        """
        Get form field for the given Django model field.

        By default ``Field.formfield()`` is used to get the form
        field unless an overwrite is present for the field.
        Overwrites are useful for non-standard fields like
        ``FileField`` since in that case ``CharField``
        should be used.
        """
        overwrite = MODEL_FIELD_OVERWRITES.get(field.__class__)
        if overwrite is not None:
            if callable(overwrite):
                return overwrite(field)
            else:
                return overwrite

        form_field = field.formfield()

        if form_field is None:
            raise SkipFilter

        return form_field

    def _build_filter(self, name, state):
        field = self.Meta.model._meta.get_field(
            self._get_filter_extra_kwargs(name).get('source', name)
        )

        if isinstance(field, RelatedField):
            if not self.Meta.allow_related:
                raise SkipFilter
            return self._build_filterset_from_related_field(name, field)

        elif isinstance(field, ForeignObjectRel):
            if not self.Meta.allow_related_reverse:
                raise SkipFilter
            return self._build_filterset_from_reverse_field(name, field)

        elif GenericForeignKey and isinstance(field, GenericForeignKey):
            raise SkipFilter

        else:
            return self._build_filter_from_field(name, field)

    def _build_filter_from_field(self, name, field):
        """
        Build :class:`.Filter` for a standard Django model field.
        """
        return Filter(
            form_field=self._get_form_field_for_field(field),
            is_default=field.primary_key,
            **self._get_filter_extra_kwargs(name)
        )

    def _build_filterset_from_related_field(self, name, field):
        """
        Build a :class:`.FilterSet` for a Django relation model field
        such as ``ForeignKey``.
        """
        # field.rel for Django < 1.9
        remote_field = getattr(field, 'remote_field', None) or field.rel

        return self._build_django_filterset(name, field, {
            'exclude': [remote_field.name],
        })

    def _build_filterset_from_reverse_field(self, name, field):
        """
        Build a :class:`.FilterSet` for a Django reverse relation model field.
        """
        return self._build_django_filterset(name, field, {
            'exclude': [field.field.name],
        })

    def _build_django_filterset(self, name, field, meta_attrs):
        m = field.related_model
        attrs = {'model': m}
        attrs.update(meta_attrs)

        return self._build_filterset(
            m.__name__,
            name,
            attrs,
            ModelFilterSet,
        )



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import mock
import pytest
from django.core.exceptions import ValidationError as DjangoValidationError
from django.http import QueryDict
from rest_framework.exceptions import ValidationError

from test_project.one_to_one.api import PlaceFilterSet
from test_project.one_to_one.models import Place, Restaurant
from url_filter.filtersets import FilterSet, ModelFilterSet
from url_filter.integrations.drf import DjangoFilterBackend


class TestDjangoFilterBackend(object):
    def test_get_filter_class_supplied(self):
        class View(object):
            filter_class = PlaceFilterSet

        filter_class = DjangoFilterBackend().get_filter_class(
            View(), Place.objects.all()
        )

        assert filter_class is PlaceFilterSet

    def test_get_filter_class_by_filter_fields(self):
        class View(object):
            filter_fields = ['name']

        filter_class = DjangoFilterBackend().get_filter_class(
            View(), Place.objects.all()
        )

        assert issubclass(filter_class, ModelFilterSet)
        assert filter_class.Meta.model is Place
        assert filter_class.Meta.fields == ['name']

    def test_get_filter_context(self):
        context = DjangoFilterBackend().get_filter_context(
            request='request', view='view',
        )

        assert context == {
            'request': 'request',
            'view': 'view',
        }

    def test_get_filter_queryset_not_filtered(self):
        assert DjangoFilterBackend().filter_queryset(None, None, None) is None

    @mock.patch.object(FilterSet, 'filter')
    def test_get_filter_queryset(self, mock_filter, db, rf):
        class View(object):
            filter_fields = ['name']

        request = rf.get('/')
        request.query_params = QueryDict()

        filtered = DjangoFilterBackend().filter_queryset(
            request=request,
            queryset=Place.objects.all(),
            view=View()
        )

        assert filtered == mock_filter.return_value

    @mock.patch.object(FilterSet, 'filter')
    def test_get_filter_queryset_invalid_query(self, mock_filter, db, rf):
        mock_filter.side_effect = DjangoValidationError({'foo': 'bar'})

        class View(object):
            filter_fields = ['name']

        request = rf.get('/')
        request.query_params = QueryDict()

        with pytest.raises(ValidationError) as e:
            DjangoFilterBackend().filter_queryset(
                request=request,
                queryset=Place.objects.all(),
                view=View()
            )

        assert e.value.detail == {'foo': ['bar']}

    @mock.patch.object(FilterSet, 'filter')
    def test_filter_queryset_supplied_model_mismatch(self, mock_filter, db, rf):
        class View(object):
            filter_class = PlaceFilterSet
            filter_fields = ['name']

        request = rf.get('/')
        request.query_params = QueryDict()

        with pytest.raises(AssertionError):
            DjangoFilterBackend().filter_queryset(
                request=request,
                queryset=Restaurant.objects.all(),
                view=View()
            )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import enum


class StrictMode(enum.Enum):
    """
    Strictness mode enum.

    :``empty``:
        when validation fails for any filter within :class:`.FilterSet`,
        empty queryset should be returned
    :``drop`` (default):
        ignores all filter failures. when any occur, :class:`.FilterSet`
        simply then does not filter provided queryset.
    :``fail``:
        when validation fails for any filter within :class:`.FilterSet`,
        all error are compiled and cumulative ``ValidationError`` is raised.
    """
    empty = 'empty'
    drop = 'drop'
    fail = 'fail'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import inspect
from functools import partial

from django import forms
from sqlalchemy.orm.properties import ColumnProperty, RelationshipProperty
from sqlalchemy.types import (
    BIGINT,
    CHAR,
    CLOB,
    DATE,
    DECIMAL,
    INTEGER,
    SMALLINT,
    TIMESTAMP,
    VARCHAR,
    BigInteger,
    Boolean,
    Date,
    DateTime,
    Float,
    Integer,
    Numeric,
    String,
)

from ..backends.sqlalchemy import SQLAlchemyFilterBackend
from ..exceptions import SkipFilter
from ..filters import Filter
from ..utils import SubClassDict
from .base import BaseModelFilterSet


__all__ = ['SQLAlchemyModelFilterSet']


def _string(field, column):
    return forms.CharField(max_length=column.type.length)


SQLALCHEMY_FIELD_MAPPING = SubClassDict({
    BIGINT: forms.IntegerField,
    BigInteger: forms.IntegerField,
    Integer: forms.IntegerField,
    Boolean: partial(forms.BooleanField, required=False),
    CHAR: _string,
    CLOB: _string,
    DATE: forms.DateTimeField,
    Date: forms.DateField,
    DateTime: forms.DateTimeField,
    DECIMAL: forms.DecimalField,
    Float: forms.FloatField,
    INTEGER: forms.IntegerField,
    Numeric: forms.IntegerField,
    SMALLINT: forms.IntegerField,
    String: _string,
    TIMESTAMP: forms.DateTimeField,
    VARCHAR: _string,
})


class SQLAlchemyModelFilterSet(BaseModelFilterSet):
    """
    :class:`.FilterSet` for SQLAlchemy models.

    The filterset can be configured via ``Meta`` class attribute,
    very much like Django's ``ModelForm`` is configured.
    """
    filter_backend_class = SQLAlchemyFilterBackend

    def _build_filter(self, name, fields):
        field = fields[self._get_filter_extra_kwargs(name).get('source', name)]

        if isinstance(field, ColumnProperty):
            return self._build_filter_from_field(name, field)

        elif isinstance(field, RelationshipProperty):
            if not self.Meta.allow_related:
                raise SkipFilter
            return self._build_filterset_from_related_field(name, field)

    def _build_state(self):
        """
        Build state of all column properties for the SQLAlchemy model
        which normalizes to a dict where keys are field names and values
        are column property instances.
        This state is computed before main loop which builds all filters
        for all fields. As a result all helper builder methods
        can use this state to get column property instances for necessary
        fields by simply doing a dictionary lookup instead of requiring
        search operations to find appropriate properties.
        """
        return SQLAlchemyFilterBackend._get_properties_for_model(self.Meta.model)

    def _get_model_field_names(self):
        """
        Get a list of all model fields.

        This is used when ``Meta.fields`` is ``None``
        in which case this method returns all model fields.
        """
        return list(SQLAlchemyFilterBackend._get_properties_for_model(self.Meta.model).keys())

    def _get_form_field_for_field(self, field):
        """
        Get form field for the given SQLAlchemy model field.
        """
        column = SQLAlchemyFilterBackend._get_column_for_field(field)

        form_field = SQLALCHEMY_FIELD_MAPPING.get(
            column.type.__class__, None,
        )

        if form_field is None:
            raise SkipFilter

        if inspect.isclass(form_field) or isinstance(form_field, partial):
            return form_field()
        else:
            return form_field(field, column)

    def _build_filter_from_field(self, name, field):
        """
        Build :class:`.Filter` for a standard SQLAlchemy model field.
        """
        column = SQLAlchemyFilterBackend._get_column_for_field(field)

        return Filter(
            form_field=self._get_form_field_for_field(field),
            is_default=column.primary_key,
            **self._get_filter_extra_kwargs(name)
        )

    def _build_filterset_from_related_field(self, name, field):
        """
        Build :class:`.FilterSet` for a relation SQLAlchemy model field.
        """
        m = SQLAlchemyFilterBackend._get_related_model_for_field(field)

        return self._build_filterset(
            m.__name__,
            name,
            {
                'model': m,
                'exclude': [field.back_populates]
            },
            SQLAlchemyModelFilterSet,
        )
